"use strict";

import { useState } from 'react';
import { Pressable } from 'react-native';
import { Canvas, RoundedRect, Shadow } from '@shopify/react-native-skia';
import { interpolate, useDerivedValue, useSharedValue, withTiming } from 'react-native-reanimated';
import { COMMON_STYLES, DEFAULT_REFLECTED_LIGHT_COLOR, DEFAULT_SHADOW_COLOR } from "../constants.js";
import { getBackgroundColor } from "../utils.js";

/**
 * ShadowPressable
 * ----------------
 * A pressable component that casts a shadow when pressed.
 * The shadow effect is created using the `@shopify/react-native-skia` library.
 *
 * @param initialDepth - The initial depth of the shadow
 * @param shadowSpace - The space between the shadow and the component
 * @param duration - The duration of the shadow animation
 * @param damping - The damping factor of the shadow animation
 * @param isReflectedLightEnabled - Whether the reflected light effect is enabled
 */
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
export const ShadowPressable = ({
  width: _width = 0,
  height: _height = 0,
  initialDepth = 3,
  shadowSpace = 6,
  shadowBlur = 10,
  shadowColor = DEFAULT_SHADOW_COLOR,
  reflectedLightColor = DEFAULT_REFLECTED_LIGHT_COLOR,
  duration = 200,
  damping = 0.8,
  isReflectedLightEnabled = true,
  ...props
}) => {
  const [boxSize, setBoxSize] = useState({
    width: _width,
    height: _height
  });

  // Determine the final background color (pulling from `props.style` or a default).
  const backgroundColor = getBackgroundColor(props);
  const boxRadius = Number(props?.style ? props.style.borderRadius : 0) || 10;
  const depth = useSharedValue(initialDepth);
  const offset = useDerivedValue(() => Math.abs(depth.value));
  const rfOffset = useDerivedValue(() => interpolate(depth.value, [-initialDepth, initialDepth * damping], [-3, 3]));
  const inset = useDerivedValue(() => depth.value <= 0);
  const blur = useDerivedValue(() => interpolate(Math.abs(depth.value), [shadowBlur, 0], [0, initialDepth * damping]));
  const onPressIn = () => {
    depth.value = withTiming(-initialDepth * damping, {
      duration
    });
  };
  const onPressOut = () => {
    depth.value = withTiming(initialDepth, {
      duration
    });
  };
  return /*#__PURE__*/_jsxs(Pressable, {
    onLayout: ({
      nativeEvent: {
        layout: {
          width,
          height
        }
      }
    }) => setBoxSize({
      width,
      height
    }),
    ...props,
    style: [props.style, COMMON_STYLES.canvasWrapper],
    onPressIn: onPressIn,
    onPressOut: onPressOut,
    children: [boxSize.width === 0 && boxSize.height === 0 ? null : /*#__PURE__*/_jsx(Canvas, {
      style: [props.style, COMMON_STYLES.canvas, {
        width: boxSize.width,
        height: boxSize.height
      }],
      children: /*#__PURE__*/_jsxs(RoundedRect, {
        x: shadowSpace,
        y: shadowSpace,
        width: boxSize.width - shadowSpace * 2,
        height: boxSize.height - shadowSpace * 2,
        r: boxRadius,
        color: backgroundColor // The background fill of the rect
        ,
        children: [/*#__PURE__*/_jsx(Shadow, {
          dx: offset,
          dy: offset,
          blur: blur,
          color: shadowColor,
          inner: inset
        }), isReflectedLightEnabled && /*#__PURE__*/_jsx(Shadow, {
          dx: rfOffset,
          dy: rfOffset,
          blur: blur,
          color: reflectedLightColor,
          inner: true
        })]
      })
    }), props.children]
  });
};
//# sourceMappingURL=ShadowPressable.js.map