"use strict";

import { StyleSheet } from 'react-native';
import { vec } from '@shopify/react-native-skia';
import { DEFAULT_BACKGROUND_COLOR, DEFAULT_REFLECTED_LIGHT_BLUR, DEFAULT_REFLECTED_LIGHT_COLOR, DEFAULT_REFLECTED_LIGHT_OFFSET_SCALE, DEFAULT_SHADOW_BLUR, DEFAULT_SHADOW_COLOR, DEFAULT_SHADOW_OFFSET_SCALE } from "./constants.js";

/**
 *  createStyles generates the StyleSheet object for the canvas
 *
 * @param width - The width of the canvas
 * @param height - The height of the canvas
 * @returns The StyleSheet object for the canvas
 */
export function createStyles({
  width,
  height
}) {
  return StyleSheet.create({
    canvas: {
      position: 'absolute',
      left: 0,
      top: 0,
      backgroundColor: 'transparent',
      width,
      height
    }
  });
}

/**
 * getBackgroundColor retrieves the final background color
 * from either:
 *   1) props.backgroundColor
 *   2) style.backgroundColor
 *   3) DEFAULT_BACKGROUND_COLOR
 *
 * This ensures there is always a valid color for the component’s background.
 */
export function getBackgroundColor(props) {
  const backgroundColor = props.backgroundColor ?? props.style?.backgroundColor ?? DEFAULT_BACKGROUND_COLOR;
  return backgroundColor;
}

/**
 * getShadowProperty determines the final configuration for both
 * the main shadow and any reflected light. It merges default values
 * with provided props to form a complete “shadow settings” object.
 *
 * - shadowOffset / reflectedLightOffset: how far the shadows/highlights
 *   are shifted in x and y.
 * - shadowColor / reflectedLightColor: colors used for each effect.
 * - shadowBlur / reflectedLightBlur: blur radius for the softness/spread
 *   of the shadow or highlight.
 *
 * @param props - The props object containing shadow-related settings.
 * @returns {
 * shadowOffset, reflectedLightOffset, shadowColor, reflectedLightColor, shadowBlur, reflectedLightBlur}
 */
export function getShadowProperty(props) {
  const SHADOW_OFFSET_WIDTH = props.shadowOffset?.width ?? DEFAULT_SHADOW_OFFSET_SCALE;
  const SHADOW_OFFSET_HEIGHT = props.shadowOffset?.height ?? DEFAULT_SHADOW_OFFSET_SCALE;

  // By default, the reflected light offset is the inverse of the main shadow
  // so it appears on the opposite corner/side.
  // when `inset` property is `true`, the reflected light offset is opposite to the shadow offset
  const REFLECTED_LIGHT_OFFSET_WIDTH = setReflectedLightDirectionAndScale({
    inset: props.inset,
    reflectedOffset: props.reflectedLightOffset?.width,
    shadowOffset: SHADOW_OFFSET_WIDTH
  });
  const REFLECTED_LIGHT_OFFSET_HEIGHT = setReflectedLightDirectionAndScale({
    inset: props.inset,
    reflectedOffset: props.reflectedLightOffset?.height,
    shadowOffset: SHADOW_OFFSET_HEIGHT
  });

  // "Blur" here maps to how soft or large the shadow/highlight is.
  // The higher the number, the more diffuse the effect.
  const shadowBlur = Math.max(props.shadowBlur ?? DEFAULT_SHADOW_BLUR, 0);
  const reflectedLightBlur = Math.max(props.reflectedLightBlur ?? DEFAULT_REFLECTED_LIGHT_BLUR, 0);

  // Fallback to the provided defaults if the user doesn't specify a color.
  const shadowColor = props.shadowColor ?? DEFAULT_SHADOW_COLOR;
  const reflectedLightColor = props.reflectedLightColor ?? DEFAULT_REFLECTED_LIGHT_COLOR;

  // Construct the final offsets as objects for clarity.
  const shadowOffset = {
    width: SHADOW_OFFSET_WIDTH,
    height: SHADOW_OFFSET_HEIGHT
  };
  const reflectedLightOffset = {
    width: REFLECTED_LIGHT_OFFSET_WIDTH,
    height: REFLECTED_LIGHT_OFFSET_HEIGHT
  };
  return {
    shadowOffset,
    reflectedLightOffset,
    shadowColor,
    reflectedLightColor,
    shadowBlur,
    reflectedLightBlur
  };
}
function setReflectedLightDirectionAndScale({
  inset,
  reflectedOffset,
  shadowOffset
}) {
  // When user provides a reflected light offset, use that.
  if (reflectedOffset !== undefined) {
    return reflectedOffset;
  }

  // When shadow is 0, reflected light should be 0.
  if (shadowOffset === 0) {
    return 0;
  }

  // When inset is true, the reflected light should be opposite the shadow.
  if (inset) {
    return -(shadowOffset * DEFAULT_REFLECTED_LIGHT_OFFSET_SCALE) / shadowOffset;
  }
  return shadowOffset * DEFAULT_REFLECTED_LIGHT_OFFSET_SCALE / shadowOffset;
}
export function createOuterShadowOffset({
  inset,
  shadowColor,
  shadowOffset,
  shadowBlur
}) {
  if (!inset) {
    return {
      shadowColor,
      shadowOffset,
      // blur: 0 ~ 20, opacity: 0 ~ 1
      shadowOpacity: shadowBlur ? shadowBlur / 5 : 0.6,
      shadowRadius: shadowBlur * 0.6,
      elevation: shadowBlur
    };
  }
  return {};
}
/**
 * `getLinearDirection` calculates the start and end points for a linear gradient
 * based on the provided direction (from, to).
 * The direction is specified as a string, e.g., 'top', 'bottom', 'left', 'right'.
 * The width and height are used to calculate the midpoints for each direction.
 */
export function getLinearDirection({
  width,
  height,
  from,
  to
}) {
  const top = vec(width / 2, 0);
  const bottom = vec(width / 2, height);
  const left = vec(0, height / 2);
  const right = vec(width, height / 2);
  const direction = {
    top,
    bottom,
    left,
    right
  };
  return {
    start: direction[from],
    end: direction[to]
  };
}

/**
 * `isLinearProps` checks if the provided props are for a linear gradient.
 * If the `colors` property is an array, we assume it's a linear gradient.
 */
export function isLinearProps(props) {
  return 'colors' in props && Array.isArray(props.colors);
}
//# sourceMappingURL=utils.js.map